@@ -1,7 +1,7 @@
 // samplv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -38,6 +38,8 @@
 #include "samplv1_controls.h"
 #include "samplv1_programs.h"
 
+#include "samplv1_sched.h"
+
 
 #ifdef CONFIG_DEBUG_0
 #include <stdio.h>
@@ -694,6 +696,38 @@ struct samplv1_voice : public samplv1_list<samplv1_voice>
 };
 
 
+// MIDI input asynchronous status notification
+
+class samplv1_midi_in : public samplv1_sched
+{
+public:
+
+	samplv1_midi_in (samplv1 *pSampl)
+		: samplv1_sched(pSampl, MidiIn),
+			m_enabled(false), m_count(0) {}
+
+	void schedule_event()
+		{ if (m_enabled && ++m_count < 2) schedule(-1); }
+
+	void process(int) {}
+
+	void enabled(bool on)
+		{ m_enabled = on; m_count = 0; }
+
+	uint32_t count()
+	{
+		const uint32_t ret = m_count;
+		m_count = 0;
+		return ret;
+	}
+
+private:
+
+	bool     m_enabled;
+	uint32_t m_count;
+};
+
+
 // polyphonic synth implementation
 
 class samplv1_impl
@@ -725,8 +759,6 @@ public:
 	void setParamValue(samplv1::ParamIndex index, float fValue);
 	float paramValue(samplv1::ParamIndex index);
 
-	void reset();
-
 	samplv1_controls *controls();
 	samplv1_programs *programs();
 
@@ -735,6 +767,13 @@ public:
 
 	bool sampleLoopTest();
 
+	void reset();
+
+	void midiInEnabled(bool on);
+	uint32_t midiInCount();
+
+	void directNoteOn(int note, int vel);
+
 	samplv1_sample  gen1_sample;
 	samplv1_wave_lf lfo1_wave;
 
@@ -777,6 +816,7 @@ private:
 	samplv1_config   m_config;
 	samplv1_controls m_controls;
 	samplv1_programs m_programs;
+	samplv1_midi_in  m_midi_in;
 
 	uint16_t m_nchannels;
 	float    m_srate;
@@ -822,6 +862,10 @@ private:
 
 	samplv1_reverb m_reverb;
 	samplv1_phasor m_phasor;
+
+	volatile int m_direct_chan;
+	volatile int m_direct_note;
+	volatile int m_direct_vel;
 };
 
 
@@ -846,8 +890,8 @@ samplv1_voice::samplv1_voice ( samplv1_impl *pImpl ) :
 // engine constructor
 
 samplv1_impl::samplv1_impl (
-	samplv1 *pSampl, uint16_t nchannels, float srate )
-	: gen1_sample(pSampl), m_controls(pSampl), m_programs(pSampl), m_bpm(180.0f)
+	samplv1 *pSampl, uint16_t nchannels, float srate ) : gen1_sample(pSampl),
+		m_controls(pSampl), m_programs(pSampl), m_midi_in(pSampl), m_bpm(180.0f)
 {
 	// null sample.
 	m_gen1.sample0 = 0.0f;
@@ -1413,6 +1457,9 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 
 	// process pending controllers...
 	m_controls.process_dequeue();
+
+	// asynchronous event notification...
+	m_midi_in.schedule_event();
 }
 
 
@@ -1461,6 +1508,8 @@ void samplv1_impl::allNotesOff (void)
 	gen1_last = 0.0f;
 
 	m_aux1.reset();
+
+	m_direct_chan = m_direct_note = m_direct_vel = -1;
 }
 
 
@@ -1484,6 +1533,36 @@ void samplv1_impl::allSustainOff (void)
 }
 
 
+// direct note-on triggered on next cycle...
+void samplv1_impl::directNoteOn ( int note, int vel )
+{
+	if (vel > 0) {
+		const int ch1 = int(*m_def.channel);
+		m_direct_chan = (ch1 > 0 ? ch1 - 1 : 0) & 0x0f;
+		m_direct_note = note;
+		m_direct_vel  = vel;
+	} else {
+		m_direct_vel  = 0;
+	}
+}
+
+
+// controllers accessor
+
+samplv1_controls *samplv1_impl::controls (void)
+{
+	return &m_controls;
+}
+
+
+// programs accessor
+
+samplv1_programs *samplv1_impl::programs (void)
+{
+	return &m_programs;
+}
+
+
 // all reset clear
 
 void samplv1_impl::reset (void)
@@ -1528,22 +1607,20 @@ void samplv1_impl::reset (void)
 }
 
 
-// controllers accessor
+// MIDI input asynchronous status notification accessors
 
-samplv1_controls *samplv1_impl::controls (void)
+void samplv1_impl::midiInEnabled ( bool on )
 {
-	return &m_controls;
+	m_midi_in.enabled(on);
 }
 
 
-// programs accessor
-
-samplv1_programs *samplv1_impl::programs (void)
+uint32_t samplv1_impl::midiInCount (void)
 {
-	return &m_programs;
+	return m_midi_in.count();
 }
 
-
+ 
 // synthesize
 
 void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
@@ -1561,6 +1638,20 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		::memset(outs[k], 0, nframes * sizeof(float));
 	}
 
+	// process direct note on/off...
+	if (m_direct_chan >= 0 && m_direct_note >= 0 && m_direct_vel >= 0) {
+		struct note_data { uint8_t status, note, vel; } data;
+		data.status = (m_direct_vel > 0 ? 0x90 : 0x80) | m_direct_chan;
+		data.note = m_direct_note;
+		data.vel = m_direct_vel;
+		process_midi((uint8_t *) &data, sizeof(data));
+		if (m_direct_vel == 0) {
+			m_direct_chan = -1;
+			m_direct_note = -1;
+		}
+		m_direct_vel = -1;
+	}
+
 	// channel indexes
 
 	const uint16_t k11 = 0;
@@ -1793,7 +1884,6 @@ bool samplv1_impl::sampleLoopTest (void)
 	return gen1_sample.loop_test(*m_gen1.loop > 0.5f);
 }
 
-
 //-------------------------------------------------------------------------
 // samplv1 - decl.
 //
@@ -1989,4 +2079,26 @@ bool samplv1::sampleLoopTest (void) const
 }
 
 
+// MIDI input asynchronous status notification accessors
+
+void samplv1::midiInEnabled ( bool on )
+{
+	m_pImpl->midiInEnabled(on);
+}
+
+
+uint32_t samplv1::midiInCount (void)
+{
+	return m_pImpl->midiInCount();
+}
+
+
+// MIDI direct note on/off triggering
+
+void samplv1::directNoteOn ( int note, int vel )
+{
+	m_pImpl->directNoteOn(note, vel);
+}
+
+
 // end of samplv1.cpp
