@@ -1,7 +1,7 @@
 // samplv1widget.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -50,8 +50,9 @@ public:
 	// Destructor.
 	virtual ~samplv1widget();
 
-	// Create/initialize the scheduler/work notifier.
-	void initSchedNotifier();
+	// Open/close the scheduler/work notifier.
+	void openSchedNotifier();
+	void closeSchedNotifier();
 
 	// Param port accessors.
 	void setParamValue(
@@ -59,8 +60,8 @@ public:
 	float paramValue(samplv1::ParamIndex index) const;
 
 	// Param kbob (widget) mapper.
-	void setParamKnob(samplv1::ParamIndex index, samplv1widget_knob *pKnob);
-	samplv1widget_knob *paramKnob(samplv1::ParamIndex index) const;
+	void setParamKnob(samplv1::ParamIndex index, samplv1widget_param *pKnob);
+	samplv1widget_param *paramKnob(samplv1::ParamIndex index) const;
 
 	// MIDI note/octave name helper.
 	static QString noteName(int note);
@@ -96,6 +97,9 @@ protected slots:
 	// Sample loader slot.
 	void loadSample(const QString& sFilename);
 
+	// Sample playback (direct note-on/off).
+	void playSample(void);
+
 	// Common context menu.
 	void contextMenuRequest(const QPoint& pos);
 
@@ -113,6 +117,9 @@ protected slots:
 	// Notification updater.
 	void updateSchedNotify(int stype, int sid);
 
+	// MIDI In LED timeout.
+	void midiInLedTimeout();
+
 	// Param knob context menu.
 	void paramContextMenu(const QPoint& pos);
 
@@ -168,6 +175,10 @@ protected:
 	// Dirty flag (overridable virtual) methods.
 	virtual void updateDirtyPreset(bool bDirtyPreset);
 
+	// Show/hide dget handlers.
+	void showEvent(QShowEvent *pShowEvent);
+	void hideEvent(QHideEvent *pHideEvent);
+
 private:
 
 	// Instance variables.
@@ -175,8 +186,8 @@ private:
 
 	samplv1widget_sched *m_sched_notifier;
 
-	QHash<samplv1::ParamIndex, samplv1widget_knob *> m_paramKnobs;
-	QHash<samplv1widget_knob *, samplv1::ParamIndex> m_knobParams;
+	QHash<samplv1::ParamIndex, samplv1widget_param *> m_paramKnobs;
+	QHash<samplv1widget_param *, samplv1::ParamIndex> m_knobParams;
 
 	float m_params_ab[samplv1::NUM_PARAMS];
 
